<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://makewebbetter.com
 * @since      1.0.0
 *
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/admin
 * @author     MakeWebBetter <https://makewebbetter.com>
 */
class Mwb_Cf7_Integration_With_Zoho_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Creating Instance of the Zoho Api class.
	 *
	 * @var      object
	 * @since    1.0.0
	 */
	public $zoho_api;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param    string $plugin_name       The name of this plugin.
	 * @param    string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		$this->zoho_api = Zcf7_Api::get_instance();

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 * @return   void
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Mwb_Cf7_Integration_With_Zoho_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Mwb_Cf7_Integration_With_Zoho_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		if ( $this->is_valid_screen() ) {

			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/mwb-cf7-integration-with-zoho-admin.min.css', array(), $this->version, 'all' );
			wp_enqueue_style( $this->plugin_name . '-select2', plugin_dir_url( __FILE__ ) . 'css/select2.min.css', array(), $this->version, 'all' );
			wp_enqueue_style( $this->plugin_name . '-tooltip', plugin_dir_url( __FILE__ ) . 'css/tooltip.css', array(), $this->version, 'all' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 * @return   void
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Mwb_Cf7_Integration_With_Zoho_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Mwb_Cf7_Integration_With_Zoho_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( $this->is_valid_screen() ) {

			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/mwb-cf7-integration-with-zoho-admin.min.js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( $this->plugin_name . '-select2', plugin_dir_url( __FILE__ ) . 'js/select2.min.js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( $this->plugin_name . '-swal2', plugin_dir_url( __FILE__ ) . 'js/sweet-alert2.js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( $this->plugin_name . '-tooltip', plugin_dir_url( __FILE__ ) . 'js/jquery.tipTip.js', array( 'jquery' ), $this->version, false );
			wp_localize_script(
				$this->plugin_name,
				'mwb_zcf7_ajax',
				array(
					'ajaxUrl'          => admin_url( 'admin-ajax.php' ),
					'feedBackLink'     => admin_url( 'admin.php?page=mwb_zoho_cf7&tab=feeds' ),
					'feedBackText'     => esc_html__( 'Back to feeds', 'mwb-cf7-integration-with-zoho' ),
					'feedFormSettings' => $this->get_feed_form_settings(),
					'feedFormFilters'  => $this->get_feeds_form_filter(),
					'isPage'           => isset( $_GET['tab'] ) ? sanitize_text_field( wp_unslash( $_GET['tab'] ) ) : '', // @codingStandardsIgnoreLine
					'notFoundError'    => esc_html__( 'Settings not found', 'mwb-cf7-integration-with-zoho' ),
					'criticalError'    => esc_html__( 'Internal server error', 'mwb-cf7-integration-with-zoho' ),
					'authSecure'       => wp_create_nonce( 'mwb_zcf7_nonce' ),
					'trashImg'         => ZOHO_CF7_INTEGRATION_URL . 'admin/images/trash.svg',
				)
			);
		}
	}

	/**
	 * Check for the screens provided by the plugin.
	 *
	 * @since     1.0.0
	 * @return    bool
	 */
	public function is_valid_screen() {

		$result = false;

		$valid_screens = array(
			'mwb_zoho_cf7',
			'mwb_zoho_feeds',
		);

		$screen = get_current_screen();

		if ( ! empty( $screen->id ) ) {

			$pages = $screen->id;

			foreach ( $valid_screens as $screen ) {
				if ( false !== strpos( $pages, $screen ) ) { // @codingStandardsIgnoreLine
					$result = true;
				}
			}
		}

		return $result;
	}

	/**
	 * Get zoho modules & save it.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public function save_zoho_modules() {

		if ( get_option( 'mwb_zcf7_modules', false ) && get_option( 'mwb_zcf7_accounts_data', false ) ) {
			return;
		}

		$zoho_api = Zcf7_Api::get_instance();
		$response = $zoho_api->get_modules_data();
		$accounts = $zoho_api->get_records_data( 'Accounts' );

		update_option( 'mwb_zcf7_modules', $response );
		update_option( 'mwb_zcf7_accounts_data', $accounts );

	}

	/**
	 * Add Zoho submenu to Contact menu.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public function mwb_zoho_cf7_submenu() {
		add_submenu_page( 'wpcf7', esc_html__( 'Zoho', 'mwb-cf7-integration-with-zoho' ), esc_html__( 'Zoho', 'mwb-cf7-integration-with-zoho' ), 'manage_options', 'mwb_zoho_cf7', array( $this, 'mwb_zoho_cf7_submenu_cb' ), 3 );
	}

	/**
	 * Zoho sub-menu callback function.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public function mwb_zoho_cf7_submenu_cb() {
		require_once ZOHO_CF7_INTEGRATION_DIRPATH . 'admin/partials/zoho-cf7-integration-admin-display.php';
	}

	/**
	 * Function to run at admin intitialization.
	 *
	 * @since     1.0.0
	 * @return    bool
	 */
	public function admin_init_process() {

		/* Authorize and redirect */
		if ( isset( $_GET['mwb_get_zcf7_code'] ) && true == $_GET['mwb_get_zcf7_code'] ) { // @codingStandardsIgnoreLine
			if ( isset( $_GET['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ) ) ) { // @codingStandardsIgnoreLine

				$client_id = ! empty( $_GET['client_id'] ) ? sanitize_text_field( wp_unslash( $_GET['client_id'] ) ) : '';
				$secret_id = ! empty( $_GET['secret_id'] ) ? sanitize_text_field( wp_unslash( $_GET['secret_id'] ) ) : '';
				$domain    = ! empty( $_GET['domain'] ) ? sanitize_text_field( wp_unslash( $_GET['domain'] ) ) : '';

				if ( empty( $client_id ) || empty( $secret_id ) || empty( $domain ) ) {
					return false;
				}

				update_option( 'mwb-zcf7-client-id', $client_id );
				update_option( 'mwb-zcf7-secret-id', $secret_id );
				update_option( 'mwb-zcf7-domain', $domain );

				$auth_url = $this->zoho_api->get_auth_code_url();
				wp_redirect( $auth_url ); // @codingStandardsIgnoreLine
			}
		}

		/* If authorized redirect to plugin page */
		if ( isset( $_GET['code'] ) && isset( $_GET['accounts-server'] ) && isset( $_GET['location'] ) ) {
			$code    = sanitize_text_field( wp_unslash( $_GET['code'] ) ); // @codingStandardsIgnoreLine
			$success = $this->zoho_api->get_refresh_token_data( $code ) ? '1' : '0';
			update_option( 'mwb_is_crm_active', $success );
			wp_safe_redirect( admin_url( 'admin.php?page=mwb_zoho_cf7' ) );
		}

		/* Download log file */
		if ( isset( $_GET['mwb_download'] ) ) { // @codingStandardsIgnoreLine
			$filename = WP_CONTENT_DIR . '/uploads/zoho-cf7-logs/mwb-zcf7-sync-log.log';
			header( 'Content-type: text/plain' );
			header( 'Content-Disposition: attachment; filename="' . basename( $filename ) . '"' );
			readfile( $filename ); // @codingStandardsIgnoreLine
			exit;
		}
	}

	/**
	 * Save hook :: Saves data of the reffered object.
	 *
	 * @param     array  $formdata   An array of form data.
	 * @param     string $obj        Which data to save Account or Setting.
	 * @since     1.0.0
	 * @return    array  An array of status and message.
	 */
	public function save_crm_settings( $formdata = array(), $obj = false ) {

		$result       = array();
		$setting_data = array();

		if ( empty( $formdata ) || ! is_array( $formdata ) ) {

			$result['error'] = array(
				'status'  => false,
				'class'   => 'error',
				'message' => esc_html__( 'No data found', 'mwb-cf7-integration-with-zoho' ),
			);

		} else {

			switch ( $obj ) {

				case 'setting':
					foreach ( $formdata as $data_key => $data_value ) {

						if ( 'email_notif' == $data_key ) { // @codingStandardsIgnoreLine

							if ( '' != $data_value && ! self::validate_email( $data_value ) ) { // @codingStandardsIgnoreLine
								$setting_data['email_notif'] = '';

								$result['email_error'] = array(
									'status'  => false,
									'class'   => 'error',
									'message' => esc_html__( 'Inavlid email', 'mwb-cf7-integration-with-zoho' ),
								);
								continue;

							}
						}

						$setting_data[ $data_key ] = $data_value;
					}

					update_option( 'mwb_zcf7_setting', $setting_data );

					$result['db_response'] = array(
						'status'  => true,
						'class'   => 'success',
						'message' => esc_html__( 'Settings saved successfully', 'mwb-cf7-integration-with-zoho' ),
					);
					break;
			}
		}

		return $result;

	}

	/**
	 * Email validation.
	 *
	 * @param      string $email E-mail to validate.
	 * @since      1.0.0
	 * @return     bool
	 */
	public static function validate_email( $email = false ) {

		if ( function_exists( 'filter_var' ) ) {

			if ( filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return true;
			}
		} elseif ( function_exists( 'is_email' ) ) {

			if ( is_email( $email ) ) {
				return true;
			}
		} else {

			if ( preg_match( '/@.+\./', $email ) ) {
				return true;
			}
		}

		return false;

	}

	/**
	 * Get feed field types.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public function get_feed_form_settings() {

		$page     = get_current_screen();
		$settings = array();

		if ( 'mwb_zoho_feeds' == $page->id ) { // @codingStandardsIgnoreLine

			$feed_id = isset( $_GET['post'] ) ? sanitize_text_field( wp_unslash( $_GET['post'] ) ) : ''; // @codingStandardsIgnoreLine

			if ( ! empty( $feed_id ) ) {

				$form_id  = get_post_meta( $feed_id, 'mwb_zcf7_form', true );
				$helper   = new Mwb_Cf7_Integration_With_Zoho_Helper();
				$response = ! empty( $form_id ) ? $helper->parse_form_fields( $form_id ) : array();

				$form_data = array();
				if ( ! empty( $response ) && is_array( $response ) ) {
					foreach ( $response as $key => $value ) {
						$form_data[ $key ] = $value['label'];
					}
				}

				$form_data = array(
					'Contact Form Fields' => $form_data,
				);

				$settings['field_type'] = array(
					'label'   => esc_html__( 'Field Type', 'mwb-cf7-integration-with-zoho' ),
					'options' => array(
						'standard_field' => esc_html__( 'Standard Fields', 'mwb-cf7-integration-with-zoho' ),
						'custom_value'   => esc_html__( 'Custom Value', 'mwb-cf7-integration-with-zoho' ),
					),
				);

				$settings['field_value'] = array(
					'label'   => __( 'Field Value', 'mwb-cf7-integration-with-zoho' ),
					'options' => $form_data,
				);

				$settings['custom_value'] = array(
					'label' => esc_html__( 'Custom Value', 'mwb-cf7-integration-with-zoho' ),
				);
			}
		}

		return $settings;
	}


	/**
	 * Tooltip image
	 *
	 * @param     string $tip Tip to display.
	 * @since     1.0.0
	 * @return    void
	 */
	public static function mwb_zcf7_tooltip( $tip ) {
		?>
			<i class="mwb_zcf7_icons mwb_zcf7_tips" data-tip="<?php echo esc_html( $tip ); ?>"><span class="dashicons dashicons-editor-help"></span></i> 
		<?php

	}

	/**
	 * Notices :: Display admin notices.
	 *
	 * @param     string $class Type of notice.
	 * @param     string $msg   Message to display.
	 * @since     1.0.0
	 * @return    void
	 */
	public static function mwb_zcf7_notices( $class = false, $msg = false ) {
		?>
			<div class="notice notice-<?php echo esc_html( $class ); ?> is-dismissible mwb-notice">
				<p><strong><?php echo esc_html( $msg ); ?></strong></p>
			</div>
		<?php
	}

	/**
	 * Feeds conditional filter options.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public static function get_feeds_form_filter() {

		$filter = array(
			'exact_match'       => esc_html__( 'Matches exactly', 'mwb-cf7-integration-with-zoho' ),
			'no_exact_match'    => esc_html__( 'Does not match exactly', 'mwb-cf7-integration-with-zoho' ),
			'contains'          => esc_html__( 'Contains (Text)', 'mwb-cf7-integration-with-zoho' ),
			'not_contains'      => esc_html__( 'Does not contain (Text)', 'mwb-cf7-integration-with-zoho' ),
			'exist'             => esc_html__( 'Exist in (Text)', 'mwb-cf7-integration-with-zoho' ),
			'not_exist'         => esc_html__( 'Does not Exists in (Text)', 'mwb-cf7-integration-with-zoho' ),
			'starts'            => esc_html__( 'Starts with (Text)', 'mwb-cf7-integration-with-zoho' ),
			'not_starts'        => esc_html__( 'Does not start with (Text)', 'mwb-cf7-integration-with-zoho' ),
			'ends'              => esc_html__( 'Ends with (Text)', 'mwb-cf7-integration-with-zoho' ),
			'not_ends'          => esc_html__( 'Does not end with (Text)', 'mwb-cf7-integration-with-zoho' ),
			'less_than'         => esc_html__( 'Less than (Text)', 'mwb-cf7-integration-with-zoho' ),
			'greater_than'      => esc_html__( 'Greater than (Text)', 'mwb-cf7-integration-with-zoho' ),
			'less_than_date'    => esc_html__( 'Less than (Date/Time)', 'mwb-cf7-integration-with-zoho' ),
			'greater_than_date' => esc_html__( 'Greater than (Date/Time)', 'mwb-cf7-integration-with-zoho' ),
			'equal_date'        => esc_html__( 'Equals (Date/Time)', 'mwb-cf7-integration-with-zoho' ),
			'empty'             => esc_html__( 'Is empty', 'mwb-cf7-integration-with-zoho' ),
			'not_empty'         => esc_html__( 'Is not empty', 'mwb-cf7-integration-with-zoho' ),
		);

		return apply_filters( 'mwb_zcf7_condition_filter', $filter );
	}

	/**
	 * Feeds Condtional html.
	 *
	 * @param     string $and_condition  The and condition of current html.
	 * @param     string $and_index      The and offset of current html.
	 * @param     string $or_index       The or offset of current html.
	 * @since     1.0.0
	 * @return    mixed
	 */
	public static function render_and_conditon( $and_condition = array(), $and_index = '1', $or_index = '' ) {

		if ( empty( $and_index ) || empty( $and_condition ) || empty( $or_index ) ) {
			return;
		}

		?>
		<div class="and-condition-filter" data-and-index=<?php echo esc_attr( $and_index ); ?> >
			<select name="condition[<?php echo esc_html( $or_index ); ?>][<?php echo esc_html( $and_index ); ?>][field]"  class="condition-form-field">
				<option value="-1" ><?php esc_html_e( 'Select Field', 'mwb-cf7-integration-with-zoho' ); ?></option>
				<?php foreach ( $and_condition['form'] as $key => $value ) : ?>
					<option value="<?php echo esc_html( $key ); ?>" <?php selected( $and_condition['field'], $key ); ?> ><?php echo esc_html( $value ); ?></option>
				<?php endforeach; ?>
			</select>
			<select name="condition[<?php echo esc_html( $or_index ); ?>][<?php echo esc_html( $and_index ); ?>][option]" class="condition-option-field">
				<option value="-1"><?php esc_html_e( 'Select Condition', 'mwb-cf7-integration-with-zoho' ); ?></option>
				<?php foreach ( self::get_feeds_form_filter() as $key => $value ) : ?>
					<option value="<?php echo esc_html( $key ); ?>" <?php selected( $and_condition['option'], $key ); ?>><?php echo esc_html( $value ); ?></option>
				<?php endforeach; ?>
			</select>
			<input type="text" name="condition[<?php echo esc_html( $or_index ); ?>][<?php echo esc_html( $and_index ); ?>][value]" class="condition-value-field" value="<?php echo esc_html( ! empty( $and_condition['value'] ) ? $and_condition['value'] : '' ); ?>" placeholder="<?php esc_html_e( 'Enter value', 'mwb-cf7-integration-with-zoho' ); ?>" >
			<?php if ( 1 != $and_index ) : // @codingStandardsIgnoreLine ?>
				<span class="dashicons dashicons-no"></span>
			<?php endif; ?>
		</div>
		<?php
	}

	/**
	 * Get all valid screens to add scripts and templates.
	 *
	 * @param     array $valid_screens An array of plugin scrrens.
	 * @since     1.0.0
	 * @return    array
	 */
	public function add_mwb_frontend_screens( $valid_screens = array() ) {

		if ( is_array( $valid_screens ) ) {

			// Push your screen here.
			array_push( $valid_screens, 'contact_page_mwb_zoho_cf7' );
		}

		return $valid_screens;
	}

	/**
	 * Get all valid slugs to add deactivate popup.
	 *
	 * @param     array $valid_screens An array of plugin scrrens.
	 * @since     1.0.0
	 * @return    array
	 */
	public function add_mwb_deactivation_screens( $valid_screens = array() ) {

		if ( is_array( $valid_screens ) ) {

			// Push your screen here.
			array_push( $valid_screens, 'mwb-cf7-integration-with-zoho-crm' );
		}

		return $valid_screens;
	}


	/**
	 * Clear sync log callback.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public function clear_sync_log() {

		$last_delete     = get_option( 'mwb-zcf7-log-last-delete', time() );
		$delete_duration = Mwb_Cf7_Integration_With_Zoho_Helper::get_settings_details( 'delete_logs' );
		if ( $last_delete < ( ( $delete_duration * 24 * 60 * 60 ) + time() ) ) {
			self::delete_sync_log();
			update_option( 'mwb-zcf7-log-last-delete', time() );
		}
	}

	/**
	 * Clear log table.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public static function delete_sync_log() {
		global $wpdb;
		$table_name     = $wpdb->prefix . 'mwb_zcf7_log';
		$log_data_query = "TRUNCATE TABLE {$table_name}"; // @codingStandardsIgnoreLine
		$wpdb->query( $log_data_query, ARRAY_A ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
	}

	/**
	 * Validate Pro version compatibility.
	 *
	 * @since     1.0.1
	 * @return    void
	 */
	public function validate_version_compatibility() {

		// When Pro version in incompatible.
		if ( 'incompatible' == Mwb_Cf7_Integration_With_Zoho_Helper::version_compatibility_check() ) { // phpcs:ignore

			set_transient( 'mwb_zoho_cf7_pro_version_incompatible', 'true' );

			// Deactivate Pro Plugin.
			add_action( 'admin_init', array( $this, 'deactivate_pro_plugin' ) );

		} elseif ( 'compatible' == Mwb_Cf7_Integration_With_Zoho_Helper::version_compatibility_check() && 'true' == get_transient( 'mwb_zoho_cf7_pro_version_incompatible' ) ) { // When Pro version in compatible and transient is set.

			delete_transient( 'mwb_zoho_cf7_pro_version_incompatible' );
		}

		if ( 'true' == get_transient( 'mwb_zoho_cf7_pro_version_incompatible' ) ) { // phpcs:ignore

			// Deactivate Pro Plugin admin notice.
			add_action( 'admin_notices', array( $this, 'deactivate_pro_admin_notice' ) );
		}
	}

	/**
	 * Deactivate Pro Plugin.
	 *
	 * @since     1.0.1
	 * @return    void
	 */
	public function deactivate_pro_plugin() {

		// To hide Plugin activated notice.
		if ( ! empty( $_GET['activate'] ) ) { //phpcs:ignore

			unset( $_GET['activate'] ); //phpcs:ignore
		}

		deactivate_plugins( 'cf7-integration-with-zoho-crm/cf7-integration-with-zoho-crm.php' );
	}

	/**
	 * Deactivate Pro Plugin admin notice.
	 *
	 * @since     1.0.1
	 * @return    void
	 */
	public function deactivate_pro_admin_notice() {

		$screen = get_current_screen();

		$valid_screens = array(
			'mwb_zoho_cf7',
			'plugins',
		);

		if ( ! empty( $screen->id ) && in_array( $screen->id, $valid_screens ) ) { // phpcs:ignore
			?>

			<div class="notice notice-error is-dismissible mwb-notice">
				<p><strong><?php esc_html_e( 'CF7 Integration with ZOHO CRM', 'mwb-cf7-integration-with-zoho' ); ?></strong> <?php esc_html_e( 'is deactivated, Please Update the PRO version as this version is outdated and will not work with the current', 'mwb-cf7-integration-with-zoho' ); ?><strong><?php esc_html_e( 'MWB CF7 Integration with ZOHO CRM', 'mwb-cf7-integration-with-zoho' ); ?></strong> <?php esc_html_e( 'Free version.', 'mwb-cf7-integration-with-zoho' ); ?></p>
			</div>

			<?php
		}
	}

}
