<?php
/**
 * The complete management for the Zoho-CF7 plugin through out the site.
 *
 * @since      1.0.0
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/includes
 * @author     MakeWebBetter <https://makewebbetter.com>
 */

/**
 * The complete management for the ajax handlers.
 *
 * @since      1.0.0
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/includes
 * @author     MakeWebBetter <https://makewebbetter.com>
 */
class Mwb_Cf7_Integration_With_Zoho_Ajax_Handler {

	/**
	 * Get default response.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public function get_default_response() {
		return array(
			'status'  => false,
			'message' => esc_html__( 'Something went wrong!!', 'mwb-cf7-integration-with-zoho' ),
		);
	}

	/**
	 * Ajax handler :: Handles all ajax callbacks.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public function mwb_zcf7_ajax_callback() {

		/* Nonce verification */
		check_ajax_referer( 'mwb_zcf7_nonce', 'nonce' );

		$event    = ! empty( $_POST['event'] ) ? sanitize_text_field( wp_unslash( $_POST['event'] ) ) : '';
		$response = $this->get_default_response();

		if ( ! empty( $event ) ) {
			$data = $this->$event( $_POST );
			if ( $data ) { // @codingStandardsIgnoreLine
				$response['status']  = true;
				$response['message'] = esc_html__( 'Success', 'mwb-cf7-integration-with-zoho' );

				$response = $this->maybe_add_data( $response, $data );
			}
		}

		wp_send_json( $response );

	}

	/**
	 * Merge additional data to response.
	 *
	 * @param     array $response   An array of response.
	 * @param     array $data       An array of data to merge in response.
	 * @since     1.0.0
	 * @return    array
	 */
	public function maybe_add_data( $response, $data ) {

		if ( is_array( $data ) ) {
			$response['data'] = $data;
		}

		return $response;
	}

	/**
	 * Referesh access tokens.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public function refresh_access_token() {

		$response        = array( 'success' => false );
		$zoho_api        = Zcf7_Api::get_instance();
		$response['msg'] = esc_html__( 'Something went wrong! Check your credentials and authorize again', 'mwb-cf7-integration-with-zoho' );

		if ( $zoho_api->renew_access_token() ) {
			$token_expiry  = Mwb_Cf7_Integration_With_Zoho_Helper::get_token_expiry_details();
			$token_message = sprintf( 'Access token will expire in %s minute(s).', $token_expiry );
			$response      = array(
				'success'       => true,
				'msg'           => esc_html__( 'Success', 'mwb-cf7-integration-with-zoho' ),
				'token_message' => $token_message,
			);
		}
		return $response;
	}

	/**
	 * Reauthorize account
	 *
	 * @since      1.0.0
	 * @return     string.
	 */
	public function reauthorize_zcf7_account() {

		$response = array( 'success' => false );
		$zoho_api = Zcf7_Api::get_instance();
		$auth_url = $zoho_api->get_auth_code_url();

		if ( ! $auth_url ) {
			$response['msg'] = esc_html__( 'Something went wrong! Check your credentials and authorize again', 'mwb-cf7-integration-with-zoho' );
			delete_option( 'mwb_is_crm_active', false );
			return $response;
		}

		$response = array(
			'success' => true,
			'url'     => $auth_url,
		);

		return $response;
	}

	/**
	 * Revoke account access.
	 *
	 * @since     1.0.0
	 * @return    bool
	 */
	public function revoke_zcf7_access() {

		delete_option( 'mwb_is_crm_active' );
		delete_option( 'mwb_zcf7_zoho_token_data' );

		return true;
	}

	/**
	 * Fetch zoho objects.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public function fetch_zoho_modules() {
		$zoho_api = Zcf7_Api::get_instance();
		$modules  = $zoho_api->get_modules_data();

		return $modules;
	}

	/**
	 * Fetch object fields.
	 *
	 * @param     array $data   An array of ajax data.
	 * @since     1.0.0
	 * @return    array
	 */
	public function fetch_module_fields( $data = array() ) {

		// Get zoho crm module fields.
		$module_data = array();
		$module      = ! empty( $data['object'] ) ? sanitize_text_field( wp_unslash( $data['object'] ) ) : '';
		$force       = ! empty( $data['force'] ) ? sanitize_text_field( wp_unslash( $data['force'] ) ) : false;
		$zoho_api    = Zcf7_Api::get_instance();

		$module_data['crm_fields'] = $zoho_api->get_module_fields( $module, $force );

		$restrict = array(
			'Lead_Source',
			'Lead_Status',
			'Phone',
			'Home_Phone',
			'Other_Phone',
			'Mobile',
			'Record_Image',
			'Account_Name',
			'Asst_Phone',
			'Fax',
		);

		if ( Mwb_Cf7_Integration_With_Zoho_Helper::is_trail_expired() ) {
			foreach ( $module_data['crm_fields']['fields'] as $key => $value ) {
				if ( array_key_exists( $value['api_name'], array_flip( $restrict ) ) ) {
					unset( $module_data['crm_fields']['fields'][ $key ] );
				}
			}
			$module_data['crm_fields']['fields'] = array_values( $module_data['crm_fields']['fields'] );
		}

		return $module_data;
	}

	/**
	 * Fetch form fields.
	 *
	 * @param     array $data   An array of ajax posted data.
	 * @since     1.0.0
	 * @return    array
	 */
	public function fetch_form_fields( $data = array() ) {

		$form_data = array();
		$form_id   = ! empty( $data['form_id'] ) ? sanitize_text_field( wp_unslash( $data['form_id'] ) ) : '';
		$helper    = new Mwb_Cf7_Integration_With_Zoho_Helper();
		$response  = $helper->parse_form_fields( $form_id );

		if ( ! empty( $response ) && is_array( $response ) ) {
			foreach ( $response as $key => $value ) {
				$form_data[ $key ] = $value['label'];

			}
		}

		$form_data = array(
			'Contact Form Fields' => $form_data,
		);

		$settings['field_type'] = array(
			'label'   => esc_html__( 'Field Type', 'mwb-cf7-integration-with-zoho' ),
			'options' => array(
				'standard_field' => esc_html__( 'Standard Fields', 'mwb-cf7-integration-with-zoho' ),
				'custom_value'   => esc_html__( 'Custom Value', 'mwb-cf7-integration-with-zoho' ),
			),
		);

		$settings['field_value'] = array(
			'label'   => esc_html__( 'Field Value', 'mwb-cf7-integration-with-zoho' ),
			'options' => $form_data,
		);

		$settings['custom_value'] = array(
			'label' => esc_html__( 'Custom Value', 'mwb-cf7-integration-with-zoho' ),
		);

		return $settings;
	}

	/**
	 * Toggle feed status.
	 *
	 * @param     array $data    An array of ajax posted data.
	 * @since     1.0.0
	 * @return    bool
	 */
	public function toggle_feed_status( $data = array() ) {

		$feed_id  = ! empty( $data['feed_id'] ) ? sanitize_text_field( wp_unslash( $data['feed_id'] ) ) : '';
		$status   = ! empty( $data['status'] ) ? sanitize_text_field( wp_unslash( $data['status'] ) ) : '';
		$response = Mwb_Cf7_Integration_With_Zoho_Helper::change_post_status( $feed_id, $status );
		return $response;
	}

	/**
	 * Trash feeds.
	 *
	 * @param     array $data    An array of ajax posted data.
	 * @since     1.0.0
	 * @return    bool
	 */
	public function trash_feeds_from_list( $data = array() ) {

		$feed_id = ! empty( $data['feed_id'] ) ? sanitize_text_field( wp_unslash( $data['feed_id'] ) ) : '';
		$trash   = wp_trash_post( $feed_id );

		if ( $trash ) {
			return true;
		}
		return false;
	}

	/**
	 * Clear sync log.
	 *
	 * @param      array $data    An array of ajax posted data.
	 * @since      1.0.0
	 * @return     array          Response array.
	 */
	public function clear_sync_log( $data = array() ) {
		Mwb_Cf7_Integration_With_Zoho_Admin::delete_sync_log();
		return array( 'success' => true );
	}

	/**
	 * Download logs.
	 *
	 * @param      array $data   An arraay of ajax posted data.
	 * @since      1.0.0
	 * @return     array         Response array.
	 */
	public function download_sync_log( $data = array() ) {

		global $wpdb;
		$table_name     = $wpdb->prefix . 'mwb_zcf7_log';
		$log_data_query = "SELECT * FROM {$table_name} ORDER BY `id` DESC"; // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$log_data       = $wpdb->get_results( $log_data_query, ARRAY_A ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$path           = Mwb_Cf7_Integration_With_Zoho_Helper::create_log_folder( 'zoho-cf7-logs' );
		$log_dir        = $path . '/mwb-zcf7-sync-log.log';

		foreach ( $log_data as $key => $value ) {

			$value['zoho_id'] = ! empty( $value['zoho_id'] ) ? $value['zoho_id'] : '-';

			$log  = 'FEED ID: ' . $value['feed_id'] . PHP_EOL;
			$log .= 'FEED : ' . $value['feed'] . PHP_EOL;
			$log .= 'ZOHO ID : ' . $value['zoho_id'] . PHP_EOL;
			$log .= 'ZOHO OBJECT : ' . $value['zoho_object'] . PHP_EOL;
			$log .= 'TIME : ' . gmdate( 'd-m-Y h:i A', esc_html( $value['time'] ) ) . PHP_EOL;
			$log .= 'REQUEST : ' . wp_json_encode( maybe_unserialize( $value['request'] ) ) . PHP_EOL;
			$log .= 'RESPONSE : ' . wp_json_encode( maybe_unserialize( $value['response'] ) ) . PHP_EOL;
			$log .= '------------------------------------' . PHP_EOL;
			file_put_contents( $log_dir, $log, FILE_APPEND ); // @codingStandardsIgnoreLine
		}

		return array(
			'success'  => true,
			'redirect' => admin_url( '?mwb_download=1' ),
		);
	}

	/**
	 * Enable datatable.
	 *
	 * @param     mixed $data    An array of ajax posted data.
	 * @since     1.0.0
	 * @return    void
	 */
	public function get_datatable_data_cb( $data = array() ) {

		$request = $_GET; // @codingStandardsIgnoreLine
		$offset  = $request['start'];
		$limit   = $request['length'];

		global $wpdb;
		$table_name     = $wpdb->prefix . 'mwb_zcf7_log';
		$log_data_query = $wpdb->prepare( "SELECT * FROM {$table_name} ORDER BY `id` DESC LIMIT %d OFFSET %d ", $limit, $offset ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$log_data       = $wpdb->get_results( $log_data_query, ARRAY_A ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$count_query    = "SELECT COUNT(*) as `total_count` FROM {$table_name}"; // @codingStandardsIgnoreLine
		$count_data     = $wpdb->get_col( $count_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$total_count    = $count_data[0];
		$data           = array();

		foreach ( $log_data as $key => $value ) {

			$data_href = Mwb_Cf7_Integration_With_Zoho_Helper::get_zoho_link( $value );
			if ( in_array( $value['zoho_object'], array( 'Note', 'Task' ) ) ) { // @codingStandardsIgnoreLine
				$link = $value['zoho_id'];
			} elseif ( ! empty( $data_href ) ) {
				$link = '<a href="' . $data_href . '" target="_blank">' . $value['zoho_id'] . '</a>';
			} else {
				$link = $value['zoho_id'];
			}

			$value['zoho_id'] = ! empty( $value['zoho_id'] ) ? $value['zoho_id'] : '-';

			$temp   = array(
				'<span class="dashicons dashicons-plus-alt"></span>',
				$value['feed'],
				$value['feed_id'],
				$value['zoho_object'],
				$link,
				$value['event'],
				gmdate( 'd-m-Y h:i A', esc_html( $value['time'] ) ),
				wp_json_encode( maybe_unserialize( $value['request'] ) ),
				wp_json_encode( maybe_unserialize( $value['response'] ) ),
			);
			$data[] = $temp;
		}

		$json_data = array(
			'draw'            => intval( $request['draw'] ),
			'recordsTotal'    => $total_count,
			'recordsFiltered' => $total_count,
			'data'            => $data,
		);

		wp_send_json( $json_data );
	}

	/**
	 * Search contact forms.
	 *
	 * @param      array $data     An array of ajax posted data.
	 * @since      1.0.1
	 * @return     void
	 */
	public function search_contact_forms( $data ) {

		$return         = array();
		$search_results = new WP_Query(
			array(
				's'                   => ! empty( $_GET['q'] ) ? sanitize_text_field( wp_unslash( $_GET['q'] ) ) : '', // phpcs:ignore
				'post_type'           => 'wpcf7_contact_form',
				'post_status'         => array( 'publish' ),
				'ignore_sticky_posts' => 1,
				'posts_per_page'      => -1,
			)
		);

		if ( $search_results->have_posts() ) {

			while ( $search_results->have_posts() ) {

				$search_results->the_post();

				$title = ( mb_strlen( $search_results->post->post_title ) > 50 ) ? mb_substr( $search_results->post->post_title, 0, 49 ) . '...' : $search_results->post->post_title;

				/**
				 * Check for post type as query sometimes returns posts even after mentioning post_type.
				 * As some plugins alter query which causes issues.
				 */
				$post_type = get_post_type( $search_results->post->ID );

				if ( 'wpcf7_contact_form' !== $post_type ) {
					continue;
				}

				$return[] = array( $search_results->post->ID, $title );

			}
		}

		wp_send_json( $return );
	}

	/**
	 * Fetch zoho accounts.
	 *
	 * @since      1.0.1
	 * @return     array
	 */
	public function refresh_account_field() {

		$zoho_api = Zcf7_Api::get_instance();
		$accounts = $zoho_api->get_records_data( 'Accounts', true );
		update_option( 'mwb_zcf7_accounts_data', $accounts ); // update accounts on account refresh.
		return $accounts;
	}

	/**
	 * Filter feeds by form
	 *
	 * @param      array $data     An array of ajax posted data.
	 * @since      1.0.0
	 * @return     mixed
	 */
	public function filter_feeds_by_form( $data ) {

		$form_id = isset( $data['form_id'] ) ? sanitize_text_field( wp_unslash( $data['form_id'] ) ) : '';

		$result = array(
			'status' => false,
			'msg'    => esc_html__( 'Invalid form', 'mwb-cf7-integration-with-zoho' ),
		);

		if ( ! empty( $form_id ) ) {

			if ( 'all' == $form_id ) { // phpcs:ignore
				$feeds = Mwb_Cf7_Integration_With_Zoho_Helper::get_feeds();
			} else {
				$feeds = Mwb_Cf7_Integration_With_Zoho_Helper::get_feeds_by_form( $form_id );
			}

			$html = '';

			foreach ( $feeds as $feed ) {
				$_status    = get_post_status( $feed->ID );
				$_form      = get_post_meta( $feed->ID, 'mwb_zcf7_form', true );
				$_object    = get_post_meta( $feed->ID, 'mwb_zcf7_object', true );
				$_prime_key = get_post_meta( $feed->ID, 'mwb_zcf7_primary_field', true );
				$checked    = 'publish' == $_status ? 'checked="checked"' : ''; // phpcs:ignore

				$html .= '<li class="mwb-zcf7__feed-row">
							<div class="mwb-zcf7__left-col">
								<h3>' . esc_html( $feed->post_title ) . '</h3>
								<div class="mwb-feed-status__wrap">
									<p class="mwb-feed-status-text_' . esc_attr( $feed->ID ) . '" ><strong>' . esc_html( 'publish' == $_status ? 'Active' : 'Sandbox' ) . '</strong></p>
								<p><input type="checkbox" class="mwb-feed-status" value="publish"' . esc_html( $checked ) . ' feed-id=' . esc_attr( $feed->ID ) . '></p>
								</div>
								<p><label><strong>' . esc_html__( 'Form : ', 'mwb-cf7-integration-with-zoho' ) . '</strong>' . esc_html( ! empty( $_form ) && '-1' != $_form ? sanitize_text_field( wp_unslash( get_the_title( $_form ) ) ) : ''  ) . '</label></p>
								<p><label><strong>' . esc_html__( 'Object : ', 'mwb-cf7-integration-with-zoho' ) . '</strong>' . esc_html( ! empty( $_object ) && '-1' != $_object ? sanitize_text_field( wp_unslash( $_object ) ) : '' ) . '</label></p>
								<p><label><strong>' . esc_html__( 'Primary key : ', 'mwb-cf7-integration-with-zoho' ) . '</strong>' . esc_html( ! empty( $_prime_key ) && '-1' != $_prime_key ? sanitize_text_field( wp_unslash( $_prime_key ) ) : '' ) . '</label></p>
							</div>
							<div class="mwb-zcf7__right-col">
								<a href="' . esc_url( get_edit_post_link( $feed->ID ) ) . '"><img src="' . esc_url( ZOHO_CF7_INTEGRATION_URL . 'admin/images/edit.svg' ) . '" alt="' . esc_html__( 'Edit feed', 'mwb-cf7-integration-with-zoho' ) .'"></a>
								<div class="mwb-zcf7__right-col1">
								<a href="javascript:void(0)" class="mwb_zcf7__trash_feed" feed-id="' . esc_html( $feed->ID ) . '"><img src="' . esc_url( ZOHO_CF7_INTEGRATION_URL . 'admin/images/trash.svg' ) . '" alt="' . esc_html__( 'Trash feed', 'mwb-cf7-integration-with-zoho' ) . '"></a>
							</div>
							</div>
						</li>';
			}

			$result = array(
				'status' => true,
				'feeds'  => $html,
			);
		}

		return $result;
	}

}
