<?php
/**
 * The plugin helper methods are defined here here.
 *
 * @since      1.0.0
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/includes
 * @author     MakeWebBetter <https://makewebbetter.com>
 */

/**
 * The complete management for the helper Objects.
 *
 * This class defines all code necessary to run helper operations.
 *
 * @since      1.0.0
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/includes
 * @author     MakeWebBetter <https://makewebbetter.com>
 */
class Mwb_Cf7_Integration_With_Zoho_Helper {

	/**
	 * Get all Contact forms.
	 *
	 * @since     1.0.0
	 * @return    array    An array of all contact forms.
	 */
	public static function get_contact_forms() {

		$args = array(
			'post_type'   => 'wpcf7_contact_form',
			'numberposts' => -1,
			'order'       => 'ASC',
		);

		return get_posts( $args );
	}

	/**
	 * Get all feeds.
	 *
	 * @since     1.0.0
	 * @return    array    An array of all feeds.
	 */
	public static function get_feeds() {

		$args = array(
			'post_type'   => 'mwb_zoho_feeds',
			'post_status' => array( 'publish', 'draft' ),
			'numberposts' => -1,
			'order'       => 'ASC',
		);

		return get_posts( $args );
	}

	/**
	 * Create log folder.
	 *
	 * @param     string $path    Name of log folder.
	 * @since     1.0.0
	 * @return    mixed
	 */
	public static function create_log_folder( $path ) {

		$basepath = WP_CONTENT_DIR . '/uploads/';
		$fullpath = $basepath . $path;

		if ( ! empty( $fullpath ) ) {

			if ( ! is_dir( $fullpath ) ) {

				$folder = mkdir( $fullpath, 0755, true );

				if ( $folder ) {
					return $fullpath;
				}
			} else {
				return $fullpath;
			}
		}
		return false;
	}

	/**
	 * Get settings of the plugin.
	 *
	 * @param      string $setting    Setting value to get.
	 * @since      1.0.0
	 * @return     string
	 */
	public static function get_settings_details( $setting = false ) {

		if ( empty( $setting ) ) {
			return false;
		}

		$result = '';
		$option = get_option( 'mwb_zcf7_setting' );

		if ( ! empty( $option ) && is_array( $option ) ) {

			switch ( $setting ) {

				case 'logs':
					$result = ! empty( $option['enable_logs'] ) ? sanitize_text_field( wp_unslash( $option['enable_logs'] ) ) : '';
					break;

				case 'delete':
					$result = ! empty( $option['data_delete'] ) ? sanitize_text_field( wp_unslash( $option['data_delete'] ) ) : '';
					break;

				case 'notif':
					$result = ! empty( $option['enable_notif'] ) ? sanitize_text_field( wp_unslash( $option['enable_notif'] ) ) : '';
					break;

				case 'email':
					$result = ! empty( $option['email_notif'] ) ? sanitize_email( wp_unslash( $option['email_notif'] ) ) : '';
					break;

				case 'delete_logs':
					$result = ! empty( $option['delete_logs'] ) ? sanitize_text_field( wp_unslash( $option['delete_logs'] ) ) : '';
					break;

				case 'dont_save_sub':
					$result = ! empty( $option['disable_save_submissions'] ) ? sanitize_text_field( wp_unslash( $option['disable_save_submissions'] ) ) : '';
					break;

				case 'dont_track':
					$result = ! empty( $option['dont_save_submission'] ) ? map_deep( wp_unslash( $option['dont_save_submission'] ), 'sanitize_text_field' ) : array();
					break;

				case 'dont_track_ip':
					$result = ! empty( $option['disable_ip_tracking'] ) ? sanitize_text_field( wp_unslash( $option['disable_ip_tracking'] ) ) : '';
			}
		}
		return $result;
	}

	/**
	 * Get token expiry details.
	 *
	 * @since     1.0.0
	 * @return    mixed
	 */
	public static function get_token_expiry_details() {

		$token_data = get_option( 'mwb_zcf7_zoho_token_data' );

		if ( $token_data['expiry'] > time() ) {
			return ceil( ( $token_data['expiry'] - time() ) / 60 );
		}
		return false;
	}

	/**
	 * Get zoho module names
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public function get_modules() {

		$data   = get_option( 'mwb_zcf7_modules', false );
		$result = array();

		if ( false == $data ) { // @codingStandardsIgnoreLine
			return false;
		}

		if ( isset( $data['modules'] ) ) {
			foreach ( $data['modules'] as $modules ) {
				if ( isset( $modules['api_supported'] ) && true == $modules['api_supported'] ) { // @codingStandardsIgnoreLine
					if ( isset( $modules['module_name'] ) && isset( $modules['api_name'] ) ) {
						$result[ $modules['api_name'] ] = $modules['module_name'];
					}
				}
			}
		}

		return $result;

	}

	/**
	 * Default settings of the plugin.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public static function mwb_zcf7_default_settings() {

		$default_settings = array(
			'enable_logs'  => 'yes',
			'data_delete'  => 'no',
			'enable_sync'  => 'yes',
			'enable_notif' => 'no',
			'email_notif'  => '',
			'delete_logs'  => 30,
		);

		return $default_settings;
	}

	/**
	 * Change post status.
	 *
	 * @param      int    $id       Post ID.
	 * @param      string $status   Status of post.
	 * @return     bool
	 */
	public static function change_post_status( $id, $status ) {

		if ( ! empty( $id ) && ! empty( $status ) ) {

			$post                = get_post( $id, 'ARRAY_A' );
			$post['post_status'] = $status;
			$response            = wp_update_post( $post );

			if ( $response && 0 != $response ) { // @codingStandardsIgnoreLine
				return true;
			}
		}
		return false;
	}

	/**
	 * Get form fields
	 *
	 * @param     int $form_id   Form ID.
	 * @since     1.0.0
	 * @return    array          An array of form fields.
	 */
	public function parse_form_fields( $form_id ) {

		$fields = array();

		$id         = $form_id;
		$form_input = get_post_meta( $id, '_form', true );

		if ( class_exists( 'WPCF7_FormTagsManager' ) ) {
			$tag_manager = WPCF7_FormTagsManager::get_instance();
			$tag_manager->scan( $form_input );
			$form_tags = $tag_manager->get_scanned_tags();

		} elseif ( class_exists( 'WPCF7_ShortcodeManager' ) ) {
			$tag_manager = WPCF7_ShortcodeManager::get_instance();
			$tag_manager->do_shortcode( $form_input );
			$form_tags = $tag_manager->get_scanned_tags();
		}

		if ( ! empty( $form_tags ) && is_array( $form_tags ) ) {

			foreach ( $form_tags as $tag ) {

				if ( is_object( $tag ) ) {
					$tag = (array) $tag;
				}

				if ( ! empty( $tag['name'] ) ) {

					$id           = str_replace( ' ', '', $tag['name'] );
					$tag['label'] = ucwords( str_replace( array( '-', '_' ), ' ', $tag['name'] ) );
					$tag['type_'] = $tag['type'];
					$tag['type']  = $tag['basetype'];
					$tag['req']   = false !== strpos( $tag['type'], '*' ) ? 'true' : '';

					if ( 'select' == $tag['type'] && ! empty( $tag['options'] ) && false !== array_search( 'multiple', $tag['options'] ) ) { // @codingStandardsIgnoreLine
						$tag['type'] = 'multiselect';
					}

					$fields[ $id ] = $tag;
				}
			}

			$fields = apply_filters( 'mwb_zcf7_form_fields', $fields );
		}

		return $fields;
	}

	/**
	 * Get form fields.
	 *
	 * @param      array $data    An array of form data.
	 * @since      1.0.0
	 * @return     array
	 */
	public static function get_form_fields( $data = array() ) {

		$form_data = array();
		if ( ! empty( $data ) && is_array( $data ) ) {
			foreach ( $data as $key => $value ) {
				$form_data[ $key ] = $value['label'];

			}
		}
		return $form_data;
	}

	/**
	 * Fetch all logs.
	 *
	 * @since     1.0.0
	 * @return    mixed
	 */
	public static function get_logs() {
		global $wpdb;
		$table_name = $wpdb->prefix . 'mwb_zcf7_log';
		$query      = "SELECT * FROM `$table_name`";
		$response   = $wpdb->get_results( $query, ARRAY_A ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		return $response;
	}

	/**
	 * Get link to data sent over zoho.
	 *
	 * @param      array $data    An array of data synced over zoho.
	 * @since      1.0.0
	 * @return     string
	 */
	public static function get_zoho_link( $data = array() ) {

		if ( empty( $data ) || ! is_array( $data ) ) {
			return;
		}

		$_domain = get_option( 'mwb-zcf7-domain', 'in' );
		$object  = $data['zoho_object'];
		$id      = $data['zoho_id'];

		$link = 'https://crm.zoho.' . $_domain . '/crm/tab/' . $object . '/' . $id;

		return $link;

	}

	/**
	 * Returns count of synced data.
	 *
	 * @since     1.0.0
	 * @return    integer
	 */
	public static function get_synced_forms_count() {

		global $wpdb;
		$table_name  = $wpdb->prefix . 'mwb_zcf7_log';
		$col_name    = 'zoho_id';
		$count_query = "SELECT COUNT(*) as `total_count` FROM {$table_name} WHERE {$col_name} != '-'"; // @codingStandardsIgnoreLine
		$count_data  = $wpdb->get_col( $count_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$total_count = isset( $count_data[0] ) ? $count_data[0] : '0';

		return $total_count;
	}

	/**
	 * Returns if pro plugin is active or not.
	 *
	 * @since      1.0.1
	 * @return     bool
	 */
	public static function pro_dependency_check() {
		// Check if pro plugin exists.
		if ( mwb_zoho_cf7_is_plugin_active( 'cf7-integration-with-zoho-crm/cf7-integration-with-zoho-crm.php' ) ) {

			if ( class_exists( 'Cf7_Integration_With_Zoho_Crm_Admin' ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Check if trial active.
	 *
	 * @since    1.0.0
	 * @return   bool
	 */
	public static function is_trail_expired() {
		$trail_over = false;
		if ( self::pro_dependency_check() ) {
			$license    = Cf7_Integration_With_Zoho_Crm::$mwb_zoho_cf7_pro_license_cb;
			$ini_days   = Cf7_Integration_With_Zoho_Crm::$mwb_zoho_cf7_pro_ini_license_cb;
			$days_count = Cf7_Integration_With_Zoho_Crm::$ini_days();

			if ( ! Cf7_Integration_With_Zoho_Crm::$license() && 0 > $days_count ) {
				$trail_over = true;
			}
		} elseif ( false === self::pro_dependency_check() ) {
			$trail_over = true;
		}
		return $trail_over;
	}

	/**
	 * Get account page links.
	 *
	 * @since  1.0.3
	 * @return array
	 */
	public static function get_zcf7_plugin_links() {

		$links = array(
			'doc'     => 'https://docs.makewebbetter.com/mwb-cf7-integration-with-zoho/?utm_source=MWB-CF7-org&utm_medium=MWB-org-backend&utm_campaign=MWB-CF7-site',
			'ticket'  => 'https://makewebbetter.com/submit-query/?utm_source=MWB-CF7-org&utm_medium=MWB-org-backend&utm_campaign=MWB-CF7-site',
			'contact' => 'https://makewebbetter.com/contact-us/?utm_source=MWB-CF7t-org&utm_medium=MWB-org-backend&utm_campaign=MWB-CF7-site',
		);

		return apply_filters( 'mwb_zcf7_plugin_links', $links );
	}

	/**
	 * Get account page links.
	 *
	 * @since  1.0.3
	 * @return array
	 */
	public static function add_plugin_headings() {

		$heading = array(
			'name'    => esc_html__( 'MWB CF7 Integration with ZOHO CRM', 'mwb-cf7-integration-with-zoho' ),
			'version' => esc_html( ZOHO_CF7_INTEGRATION_VERSION ),
		);

		return apply_filters( 'mwb_zcf7_plugin_headings', $heading );
	}


	/**
	 * Checks Whether Pro version is compatible or not.
	 *
	 * @since      1.0.1
	 * @return     mixed
	 */
	public static function version_compatibility_check() {

		if ( self::pro_dependency_check() ) {

			// When Pro plugin is outdated.
			if ( defined( 'ZOHO_CF7_INTEGRATION_PRO_VERSION' ) && version_compare( ZOHO_CF7_INTEGRATION_PRO_VERSION, '1.0.0' ) < 0 ) {

				return 'incompatible';
			} else {

				return 'compatible';
			}
		}

		return false;
	}

	/**
	 * Get feeds by form.
	 *
	 * @param     integer $form_id     Form ID.
	 * @since     1.0.1
	 * @return    array
	 */
	public static function get_feeds_by_form( $form_id = false ) {
		if ( false == $form_id ) { // phpcs:ignore
			return;
		}

		$feeds = get_posts(
			array(
				'post_type'   => 'mwb_zoho_feeds',
				'post_status' => array( 'publish', 'draft' ),
				'numberposts' => -1,
				'meta_query'  => array( // @codingStandardsIgnoreLine
					array(
						'relation' => 'AND',
						array(
							'key'     => 'mwb_zcf7_form',
							'compare' => 'EXISTS',
						),
						array(
							'key'     => 'mwb_zcf7_form',
							'value'   => $form_id,
							'compare' => '=',
						),
					),
				),
			)
		);

		return $feeds;
	}

}
